import { __decorate, __metadata } from "tslib";
import { LitElement } from 'lit';
import { property, query } from 'lit/decorators.js';
import { html } from 'lit/static-html.js';
import { classMap } from 'lit/directives/class-map.js';
import { when } from 'lit/directives/when.js';
import { INPUT_SIZES, THEME_COLORS, } from '../../internal/constants/styleConstants.js';
import { customElement } from 'lit/decorators.js';
import { BUTTON_TYPES } from '../button/constants/ButtonConstants.js';
import styles from './arc-button-group.styles.js';
import ArcDropdown from '../dropdown/ArcDropdown.js';
import ArcButton from '../button/ArcButton.js';
import '../button/arc-button.js';
import '../spinner/arc-spinner.js';
/**
 * An error that is thrown when the slot contains incorrect elements.
 * @type {Error}
 * @constant
 */
const SLOT_ERROR = new Error('arc-button-group slot error');
/**
 * The button group types.
 * @type {ButtonGroupTypes}
 * @constant
 */
export const BUTTON_GROUP_TYPES = {
    'group-filled': 'group-filled',
    'group-outlined': 'group-outlined',
};
/**
 * A button group component that groups buttons together.
 * @element arc-button-group
 * @slot - The default slot where buttons are placed.
 * @slot menu - The slot where the dropdown menu is placed.
 */
let ArcButtonGroup = class ArcButtonGroup extends LitElement {
    constructor() {
        super(...arguments);
        /**
         * The orientation of the button group.
         * @attr {boolean} column
         * @type {boolean}
         * @default false
         */
        this.column = false;
        /**
         * The color of the button group.
         * @attr {default|primary|secondary|error|warning|info|success} color
         * @type {ThemeColor}
         * @default default
         */
        this.color = THEME_COLORS.default;
        /**
         * The size of the button group.
         * @attr {small|medium|large} size
         * @type {InputSize}
         * @default medium
         */
        this.size = INPUT_SIZES.medium;
        /**
         * The type of the button group.
         * @attr {group-filled|group-outlined} type
         * @type {ButtonGroupType}
         * @default group-filled
         */
        this.type = BUTTON_GROUP_TYPES['group-filled'];
        /**
         * The disabled state of the button group.
         * @attr {boolean} disabled
         * @type {boolean}
         * @default false
         */
        this.disabled = false;
        /**
         * The loading state of the button group.
         * @attr {boolean} loading
         * @type {boolean}
         * @default false
         */
        this.loading = false;
    }
    /**
     * Ensures that the menu slot contains correct elements.
     * @throws {Error} - If the menu slot contains incorrect elements.
     * @private
     * @internal
     */
    ensureMenu() {
        var _a;
        if (!this.menu.assignedElements().length)
            return;
        if (this.menu.assignedElements().some((el) => !(el instanceof ArcDropdown)) ||
            this.menu.assignedElements().length !== 1 ||
            !((_a = this.menu.assignedElements()[0]) === null || _a === void 0 ? void 0 : _a.querySelector('arc-button'))) {
            throw SLOT_ERROR;
        }
    }
    /**
     * Updates the menu slot.
     * @param {Map<string | number | symbol, unknown>} props - Changed properties.
     * @private
     * @internal
     */
    updateMenu(props) {
        var _a;
        if (!this.menu.assignedElements().length)
            return;
        const els = (_a = this.menu) === null || _a === void 0 ? void 0 : _a.assignedElements({ flatten: true });
        if (!els)
            return;
        const dropdown = els[0];
        dropdown.setAttribute('hoist', '');
        const button = dropdown.querySelector('arc-button');
        if (!button)
            return;
        props.forEach((_, propName) => {
            if (typeof this[propName] === 'string') {
                const prop = propName;
                button.setAttribute(prop, this[prop]);
            }
            if (typeof this[propName] === 'boolean') {
                const prop = propName;
                if (this[prop]) {
                    button.setAttribute(propName, '');
                }
                else {
                    button.removeAttribute(propName);
                }
            }
        });
        const menuButtonType = this.type === BUTTON_GROUP_TYPES['group-filled']
            ? BUTTON_TYPES['group-filled-menu']
            : BUTTON_TYPES['group-outlined-menu'];
        button.setAttribute('type', menuButtonType);
    }
    /**
     * Ensures that the button group contains correct elements.
     * @throws {Error} - If the button group does not contain arc-button-group-button elements.
     * @throws {Error} - If the button group contains incorrect elements.
     * @private
     * @internal
     */
    ensureButtonGroupButtons() {
        if (!this.buttons ||
            !this.buttons.assignedElements().length ||
            this.buttons.assignedElements().some((el) => !(el instanceof ArcButton))) {
            throw SLOT_ERROR;
        }
    }
    /**
     * Updates the button group buttons.
     * @param {Map<string | number | symbol, unknown>} props - Changed properties.
     * @private
     * @internal
     */
    updateButtons(props) {
        var _a;
        const els = (_a = this.buttons) === null || _a === void 0 ? void 0 : _a.assignedElements({ flatten: true });
        if (!els)
            return;
        els.forEach((el) => {
            props.forEach((_, propName) => {
                if (typeof this[propName] === 'string') {
                    const prop = propName;
                    el.setAttribute(prop, this[prop]);
                }
                if (typeof this[propName] === 'boolean') {
                    const prop = propName;
                    if (this[prop]) {
                        el.setAttribute(propName, '');
                    }
                    else {
                        el.removeAttribute(propName);
                    }
                }
            });
        });
    }
    firstUpdated(props) {
        var _a;
        this.ensureButtonGroupButtons();
        this.updateButtons(props);
        (_a = this.buttons) === null || _a === void 0 ? void 0 : _a.addEventListener('slotchange', () => {
            this.ensureButtonGroupButtons();
            this.updateButtons(props);
        });
        this.ensureMenu();
        this.updateMenu(props);
        this.menu.addEventListener('slotchange', () => {
            this.ensureMenu();
            this.updateMenu(props);
        });
    }
    updated(props) {
        this.updateButtons(props);
        this.updateMenu(props);
    }
    render() {
        return html `
      <div
        class=${classMap({
            'button-group': true,
            'button-group--horizontal': this.column === false,
            'button-group--vertical': this.column === true,
            'button-group--small': this.size === INPUT_SIZES.small,
            'button-group--medium': this.size === INPUT_SIZES.medium,
            'button-group--large': this.size === INPUT_SIZES.large,
            'button-group--filled': this.type === BUTTON_GROUP_TYPES['group-filled'],
            'button-group--outlined': this.type === BUTTON_GROUP_TYPES['group-outlined'],
            'button-group--loading': this.loading,
        })}
      >
        <slot></slot>
        <slot id="menu" name="menu"></slot>
        ${when(this.loading, () => html `<arc-spinner id="loader"></arc-spinner>`)}
      </div>
    `;
    }
};
ArcButtonGroup.tag = 'arc-button-group';
ArcButtonGroup.styles = styles;
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcButtonGroup.prototype, "column", void 0);
__decorate([
    property({ type: String, reflect: true }),
    __metadata("design:type", String)
], ArcButtonGroup.prototype, "color", void 0);
__decorate([
    property({ type: String, reflect: true }),
    __metadata("design:type", String)
], ArcButtonGroup.prototype, "size", void 0);
__decorate([
    property({ type: String, reflect: true }),
    __metadata("design:type", String)
], ArcButtonGroup.prototype, "type", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcButtonGroup.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcButtonGroup.prototype, "loading", void 0);
__decorate([
    query('slot'),
    __metadata("design:type", HTMLSlotElement)
], ArcButtonGroup.prototype, "buttons", void 0);
__decorate([
    query('#menu'),
    __metadata("design:type", HTMLSlotElement)
], ArcButtonGroup.prototype, "menu", void 0);
ArcButtonGroup = __decorate([
    customElement('arc-button-group')
], ArcButtonGroup);
export default ArcButtonGroup;
//# sourceMappingURL=arc-button-group.js.map