import { __awaiter, __decorate, __metadata } from "tslib";
import { html, LitElement } from 'lit';
import { property, query } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';
import { HasSlotController } from '../../internal/slot.js';
import { watch } from '../../internal/watch.js';
import { getAnimation, setDefaultAnimation, startAnimations, stopAnimations, shimKeyframesHeightAuto, } from '../../internal/animate.js';
import { emit, waitForEvent } from '../../internal/event.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import { ARC_ANIMATION_OPTIONS } from '../../internal/constants/animationConstants.js';
import styles from './arc-card.styles.js';
/**
 * @slot default - The card's body.
 * @slot header - The card's header.
 * @slot image - The card's image.
 * @slot footer - The card's footer.
 *
 * @event arc-show - Emitted when the card expands.
 * @event arc-after-show - Emitted after the cards expands and all animations are complete.
 * @event arc-hide - Emitted when the card collapses.
 * @event arc-after-hide - Emitted after the card collapses and all animations are complete.
 *
 * @ssr - True
 */
export default class ArcCard extends LitElement {
    constructor() {
        super(...arguments);
        /** @internal - Controller that listens to slot changes within the component. */
        this.hasSlotController = new HasSlotController(this, 'header', 'image', '[default]', 'footer');
        /** Indicates whether the card is collapsed. This can be used instead of the expand/collapse methods. Ignored when the `header` slot is not filled. */
        this.collapsed = false;
    }
    handleCollapsedChange() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.collapsed) {
                emit(this, ARC_EVENTS.show);
                yield stopAnimations(this);
                this.content.hidden = false;
                const cardAnimation = getAnimation(this, 'card.expand');
                yield startAnimations(this.content, shimKeyframesHeightAuto(cardAnimation.keyframes, this.content.scrollHeight), cardAnimation.options);
                this.content.style.height = 'auto';
                emit(this, ARC_EVENTS.afterShow);
            }
            else {
                /* Prevent collapsing when there is no header slot. */
                if (!this.hasSlotController.test('header')) {
                    return;
                }
                emit(this, ARC_EVENTS.hide);
                yield stopAnimations(this);
                const { keyframes, options } = getAnimation(this, 'card.collapse');
                yield startAnimations(this.content, shimKeyframesHeightAuto(keyframes, this.content.scrollHeight), options);
                this.content.hidden = true;
                this.content.style.height = 'auto';
                emit(this, ARC_EVENTS.afterHide);
            }
        });
    }
    firstUpdated() {
        if (!this.hasSlotController.test('header')) {
            return;
        }
        this.content.hidden = this.collapsed;
    }
    /* Expand the card. Only when the header slot is filled. */
    expand() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.collapsed) {
                return undefined;
            }
            this.collapsed = false;
            return waitForEvent(this, ARC_EVENTS.afterShow);
        });
    }
    /* Collapse the card. Only when the header slot is filled. */
    collapse() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.collapsed || !this.hasSlotController.test('header')) {
                return undefined;
            }
            this.collapsed = true;
            return waitForEvent(this, ARC_EVENTS.afterHide);
        });
    }
    render() {
        return html `
      <article
        id="main"
        class=${classMap({
            card: true,
            'card--has-header': this.hasSlotController.test('header'),
            'card--has-image': this.hasSlotController.test('image'),
            'card--has-body': this.hasSlotController.test('[default]'),
            'card--has-footer': this.hasSlotController.test('footer'),
        })}
      >
        <header id="header">
          <slot name="header"></slot>
        </header>
        <div id="content">
          <div id="image">
            <slot name="image"></slot>
          </div>
          <div id="body">
            <slot></slot>
          </div>
          <footer id="footer">
            <slot name="footer"></slot>
          </footer>
        </div>
      </article>
    `;
    }
}
/** @internal */
ArcCard.tag = 'arc-card';
ArcCard.styles = styles;
__decorate([
    query('#content'),
    __metadata("design:type", HTMLElement)
], ArcCard.prototype, "content", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcCard.prototype, "collapsed", void 0);
__decorate([
    watch('collapsed', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArcCard.prototype, "handleCollapsedChange", null);
setDefaultAnimation('card.expand', {
    keyframes: [
        { height: '0', opacity: '0' },
        { height: 'auto', opacity: '1' },
    ],
    options: ARC_ANIMATION_OPTIONS.slow,
});
setDefaultAnimation('card.collapse', {
    keyframes: [
        { height: 'auto', opacity: '1' },
        { height: '0', opacity: '0' },
    ],
    options: ARC_ANIMATION_OPTIONS.slow,
});
//# sourceMappingURL=ArcCard.js.map