import { __decorate, __metadata } from "tslib";
import { html, LitElement } from 'lit';
import { property, query } from 'lit/decorators.js';
import { ifDefined } from 'lit/directives/if-defined.js';
import { live } from 'lit/directives/live.js';
import { classMap } from 'lit/directives/class-map.js';
import { FormController } from '../../internal/form-control.js';
import { watch } from '../../internal/watch.js';
import { emit } from '../../internal/event.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import styles from './arc-checkbox.styles.js';
/**
 * @slot default - The radio's label.
 *
 * @event arc-change - Emitted when the control's checked state changes.
 */
export default class ArcCheckbox extends LitElement {
    constructor() {
        super(...arguments);
        /** @internal - Controller used to recognize form controls located inside a shadow root. */
        /* @ts-expect-error - Controller used to hook the component to the formData */
        this.formController = new FormController(this, {
            value: (control) => control.checked ? control.value : undefined,
        });
        /** Draws the component in a disabled state. */
        this.disabled = false;
        /** Makes the chechbox a required field. */
        this.required = false;
        /** Draws the component in a checked state. */
        this.checked = false;
        /**
         * This will be true when the control is in an invalid state. Validity in radios is determined by the message provided
         * by the `setCustomValidity` method.
         */
        this.invalid = false;
    }
    firstUpdated() {
        this.invalid = !this.input.checkValidity();
    }
    /* Enable/disable the editor when the disabled property changes */
    handleDisabledChange() {
        /* Disabled form controls are always valid, so we need to recheck validity when the state changes */
        this.input.disabled = this.disabled;
        this.invalid = !this.input.checkValidity();
    }
    handleCheckedChange() {
        this.input.checked = this.checked;
        this.invalid = !this.input.checkValidity();
    }
    /* Simulates a click on the checkbox. */
    click() {
        this.input.click();
    }
    /* Sets focus on the checkbox. */
    focus(options) {
        this.input.focus(options);
    }
    /* Removes focus from the checkbox. */
    blur() {
        this.input.blur();
    }
    /** Checks for validity and shows the browser's validation message if the control is invalid. */
    reportValidity() {
        return this.input.reportValidity();
    }
    /** Sets a custom validation message. If `message` is not empty, the field will be considered invalid. */
    setCustomValidity(message) {
        this.input.setCustomValidity(message);
        this.invalid = !this.input.checkValidity();
    }
    /* Handle the click of the checkbox */
    handleClick() {
        if (!this.checked) {
            this.checked = true;
            emit(this, ARC_EVENTS.change);
        }
        else {
            this.checked = false;
            emit(this, ARC_EVENTS.change);
        }
    }
    render() {
        return html `
      <label
        id="main"
        class=${classMap({
            checkbox: true,
            'checkbox--checked': this.checked,
            'checkbox--disabled': this.disabled,
        })}
      >
        <input
          type="checkbox"
          name=${ifDefined(this.name || undefined)}
          .value=${ifDefined(this.value || undefined)}
          .checked=${live(this.checked)}
          .disabled=${this.disabled}
          .required=${this.required}
          aria-checked=${this.checked ? 'true' : 'false'}
          aria-disabled=${this.disabled ? 'true' : 'false'}
          @click=${this.handleClick}
        />
        <span id="control">
          <span id="icon">
            <svg
              class="bg"
              focusable="false"
              aria-hidden="true"
              viewBox="9 10 24 24"
              data-testid="CheckboxButtonUncheckedIcon"
            >
              <path
                d="M28.5 12.1875H13.5C13.1519 12.1875 12.8181 12.3258 12.5719 12.5719C12.3258 12.8181 12.1875 13.1519 12.1875 13.5V28.5C12.1875 28.8481 12.3258 29.1819 12.5719 29.4281C12.8181 29.6742 13.1519 29.8125 13.5 29.8125H28.5C28.8481 29.8125 29.1819 29.6742 29.4281 29.4281C29.6742 29.1819 29.8125 28.8481 29.8125 28.5V13.5C29.8125 13.1519 29.6742 12.8181 29.4281 12.5719C29.1819 12.3258 28.8481 12.1875 28.5 12.1875ZM28.6875 28.5C28.6875 28.5497 28.6677 28.5974 28.6326 28.6326C28.5974 28.6677 28.5497 28.6875 28.5 28.6875H13.5C13.4503 28.6875 13.4026 28.6677 13.3674 28.6326C13.3323 28.5974 13.3125 28.5497 13.3125 28.5V13.5C13.3125 13.4503 13.3323 13.4026 13.3674 13.3674C13.4026 13.3323 13.4503 13.3125 13.5 13.3125H28.5C28.5497 13.3125 28.5974 13.3323 28.6326 13.3674C28.6677 13.4026 28.6875 13.4503 28.6875 13.5V28.5Z"
                fill-opacity="0.6"
              />
            </svg>
            <svg
              class="fill"
              focusable="false"
              aria-hidden="true"
              viewBox="9 10 24 24"
              data-testid="CheckboxButtonCheckedIcon"
            >
              <path
                d="M28.5 12H13.5C13.1022 12 12.7206 12.158 12.4393 12.4393C12.158 12.7206 12 13.1022 12 13.5V28.5C12 28.8978 12.158 29.2794 12.4393 29.5607C12.7206 29.842 13.1022 30 13.5 30H28.5C28.8978 30 29.2794 29.842 29.5607 29.5607C29.842 29.2794 30 28.8978 30 28.5V13.5C30 13.1022 29.842 12.7206 29.5607 12.4393C29.2794 12.158 28.8978 12 28.5 12ZM25.2806 19.2806L20.0306 24.5306C19.961 24.6004 19.8783 24.6557 19.7872 24.6934C19.6962 24.7312 19.5986 24.7506 19.5 24.7506C19.4014 24.7506 19.3038 24.7312 19.2128 24.6934C19.1217 24.6557 19.039 24.6004 18.9694 24.5306L16.7194 22.2806C16.5786 22.1399 16.4996 21.949 16.4996 21.75C16.4996 21.551 16.5786 21.3601 16.7194 21.2194C16.8601 21.0786 17.051 20.9996 17.25 20.9996C17.449 20.9996 17.6399 21.0786 17.7806 21.2194L19.5 22.9397L24.2194 18.2194C24.2891 18.1497 24.3718 18.0944 24.4628 18.0567C24.5539 18.019 24.6515 17.9996 24.75 17.9996C24.8485 17.9996 24.9461 18.019 25.0372 18.0567C25.1282 18.0944 25.2109 18.1497 25.2806 18.2194C25.3503 18.2891 25.4056 18.3718 25.4433 18.4628C25.481 18.5539 25.5004 18.6515 25.5004 18.75C25.5004 18.8485 25.481 18.9461 25.4433 19.0372C25.4056 19.1282 25.3503 19.2109 25.2806 19.2806Z"
              ></path>
            </svg>
          </span>
        </span>
        <span id="label"><slot></slot></span>
      </label>
    `;
    }
}
/** @internal - The component's tag */
ArcCheckbox.tag = 'arc-checkbox';
/** @internal - The component's styles */
ArcCheckbox.styles = styles;
__decorate([
    query('input[type="checkbox"]'),
    __metadata("design:type", HTMLInputElement)
], ArcCheckbox.prototype, "input", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcCheckbox.prototype, "name", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcCheckbox.prototype, "value", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcCheckbox.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Object)
], ArcCheckbox.prototype, "required", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcCheckbox.prototype, "checked", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Object)
], ArcCheckbox.prototype, "invalid", void 0);
__decorate([
    watch('disabled', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcCheckbox.prototype, "handleDisabledChange", null);
__decorate([
    watch('checked', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcCheckbox.prototype, "handleCheckedChange", null);
//# sourceMappingURL=ArcCheckbox.js.map