import { __awaiter, __decorate, __metadata } from "tslib";
import { html, LitElement } from 'lit';
import { property, query } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';
import { autoUpdate, computePosition, flip, offset, shift, size, } from '@floating-ui/dom';
import { setDefaultAnimation, getAnimation, startAnimations, stopAnimations, } from '../../internal/animate.js';
import { emit, waitForEvent } from '../../internal/event.js';
import { watch } from '../../internal/watch.js';
import { getTabbableBoundary } from '../../internal/tabbable.js';
import { FLOATING_PLACEMENTS } from '../../internal/constants/placementConstants.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import { ARC_ANIMATION_OPTIONS } from '../../internal/constants/animationConstants.js';
import styles from './arc-dropdown.styles.js';
/**
 * @slot default - The dropdown content.
 * @slot trigger - The dropdown trigger, usually an `<arc-button>` element.
 *
 * @event arc-show - Emitted when the dropdown opens.
 * @event arc-after-show - Emitted after the dropdown opens and all animations are complete.
 * @event arc-hide - Emitted when the dropdown closes.
 * @event arc-after-hide - Emitted after the dropdown closes and all animations are complete.
 *
 * @ssr - True
 */
export default class ArcDropdown extends LitElement {
    constructor() {
        super(...arguments);
        /** The preferred placement of the dropdown panel. */
        this.placement = FLOATING_PLACEMENTS['bottom-start'];
        /** The distance in pixels from which to offset the panel away from its trigger. */
        this.distance = 0;
        /** The distance in pixels from which to offset the panel along its trigger. */
        this.skidding = 0;
        /** Indicates whether the dropdown is open. This can be used instead of the show/hide methods. */
        this.open = false;
        /** Disables the dropdown so the panel will not open. */
        this.disabled = false;
        /** Enable this option to prevent the panel from being clipped when the component is placed inside a container with overflow: auto|hidden|scroll. */
        this.hoist = false;
    }
    handleOpenChange() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.disabled) {
                return;
            }
            this._updateAccessibleTrigger();
            if (this.open) {
                /* Show */
                emit(this, ARC_EVENTS.show);
                this.addOpenListeners();
                yield stopAnimations(this);
                this.startPositioner();
                this.panel.hidden = false;
                const { keyframes, options } = getAnimation(this, 'dropdown.show');
                yield startAnimations(this.panel, keyframes, options);
                emit(this, ARC_EVENTS.afterShow);
            }
            else {
                /* Hide */
                emit(this, ARC_EVENTS.hide);
                this.removeOpenListeners();
                yield stopAnimations(this);
                const { keyframes, options } = getAnimation(this, 'dropdown.hide');
                yield startAnimations(this.panel, keyframes, options);
                this.panel.hidden = true;
                this.stopPositioner();
                emit(this, ARC_EVENTS.afterHide);
            }
        });
    }
    handlePopoverOptionsChange() {
        this.updatePositioner();
    }
    handleDisabledChange() {
        if (this.disabled && this.open) {
            this.hide();
        }
    }
    connectedCallback() {
        super.connectedCallback();
        this.handlePanelSelect = this.handlePanelSelect.bind(this);
        this.handleDocumentKeyDown = this.handleDocumentKeyDown.bind(this);
        this.handleDocumentMouseDown = this.handleDocumentMouseDown.bind(this);
        if (!this.containingElement) {
            this.containingElement = this;
        }
    }
    firstUpdated() {
        return __awaiter(this, void 0, void 0, function* () {
            this.panel.hidden = !this.open;
            /* If the dropdown is visible on init, update its position. */
            if (this.open) {
                yield this.updateComplete;
                this.addOpenListeners();
                this.startPositioner();
            }
        });
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.removeOpenListeners();
        this.hide();
        this.stopPositioner();
    }
    startPositioner() {
        this.stopPositioner();
        this.updatePositioner();
        this._positionerCleanup = autoUpdate(this.trigger, this.positioner, this.updatePositioner.bind(this));
    }
    updatePositioner() {
        if (!this.open || !this.trigger || !this.positioner) {
            return;
        }
        computePosition(this.trigger, this.positioner, {
            placement: this.placement,
            middleware: [
                offset({ mainAxis: this.distance, crossAxis: this.skidding }),
                flip(),
                shift(),
                size({
                    apply: ({ width, height }) => {
                        /* Ensure the panel stays within the viewport when we have lots of menu items */
                        Object.assign(this.panel.style, {
                            maxWidth: `${width}px`,
                            maxHeight: `${height}px`,
                        });
                    },
                    padding: 8,
                }),
            ],
            strategy: this.hoist ? 'fixed' : 'absolute',
        }).then(({ x, y, placement }) => {
            this.positioner.setAttribute('data-placement', placement);
            Object.assign(this.positioner.style, {
                position: this.hoist ? 'fixed' : 'absolute',
                left: `${x}px`,
                top: `${y}px`,
            });
        });
    }
    stopPositioner() {
        if (this._positionerCleanup) {
            this._positionerCleanup();
            this._positionerCleanup = undefined;
            this.positioner.removeAttribute('data-placement');
        }
    }
    /* Link the trigger to the dropdown panel with `aria-haspopup` and `aria-expanded`.
    These must be applied to the "accessible trigger" so screen readers will understand them.
    The accessible trigger could be the slotted element, a child of the slotted element, or an element in the slotted element's shadow root.
    For example, the accessible trigger of an <arc-button> is a <button> located inside its shadow root.
    To determine this, we assume the first tabbable element in the trigger slot is the "accessible trigger."
    */
    _updateAccessibleTrigger() {
        if (this.trigger) {
            const assignedElements = this.triggerSlot.assignedElements({
                flatten: true,
            });
            const accessibleTrigger = assignedElements.find((el) => getTabbableBoundary(el).start);
            if (accessibleTrigger) {
                accessibleTrigger.setAttribute('aria-haspopup', 'true');
                accessibleTrigger.setAttribute('aria-expanded', this.open ? 'true' : 'false');
            }
        }
    }
    addOpenListeners() {
        document.addEventListener('keydown', this.handleDocumentKeyDown);
        document.addEventListener('mousedown', this.handleDocumentMouseDown);
    }
    removeOpenListeners() {
        document.removeEventListener('keydown', this.handleDocumentKeyDown);
        document.removeEventListener('mousedown', this.handleDocumentMouseDown);
    }
    /* Shows the dropdown. */
    show() {
        if (this.open) {
            return undefined;
        }
        this.open = true;
        return waitForEvent(this, ARC_EVENTS.afterShow);
    }
    /* Hides the dropdown. */
    hide() {
        if (!this.open) {
            return undefined;
        }
        this.open = false;
        return waitForEvent(this, ARC_EVENTS.afterHide);
    }
    getMenu() {
        const slot = this.panel.querySelector('slot');
        return slot
            .assignedElements({ flatten: true })
            .filter((el) => el.tagName === 'ARC-MENU')[0];
    }
    focusOnTrigger() {
        const trigger = this.triggerSlot.assignedElements({ flatten: true })[0];
        if (trigger && typeof trigger.focus === 'function') {
            trigger.focus();
        }
    }
    handleDocumentKeyDown(event) {
        var _a, _b, _c;
        /* Close when escape is pressed */
        if (event.key === 'Escape') {
            this.hide();
            this.focusOnTrigger();
            return;
        }
        /* Handle tabbing */
        if (event.key === 'Tab') {
            const activeElement = ((_a = this.containingElement) === null || _a === void 0 ? void 0 : _a.getRootNode()) instanceof ShadowRoot
                ? (_c = (_b = document.activeElement) === null || _b === void 0 ? void 0 : _b.shadowRoot) === null || _c === void 0 ? void 0 : _c.activeElement
                : document.activeElement;
            /* Tabbing within an open menu should close the dropdown and refocus the trigger */
            if (this.open && (activeElement === null || activeElement === void 0 ? void 0 : activeElement.tagName) === 'ARC-MENU-ITEM') {
                event.preventDefault();
                this.hide();
                this.focusOnTrigger();
            }
        }
    }
    /* Close when clicking outside the containing element */
    handleDocumentMouseDown(event) {
        const path = event.composedPath();
        if (this.containingElement && !path.includes(this.containingElement)) {
            this.hide();
        }
    }
    /* Actions when the trigger has focus */
    handleTriggerKeyDown(event) {
        /* Close when escape or tab is pressed */
        if (event.key === 'Escape') {
            this.focusOnTrigger();
            this.hide();
            return;
        }
        /*
        When space bar/enter is pressed, show the panel but don't focus on the menu.
        This lets the user press the same key again to hide the menu in case they don't want to make a selection.
        */
        if ([' ', 'Enter'].includes(event.key)) {
            event.preventDefault();
            this.handleTriggerClick();
            return;
        }
        const menu = this.getMenu();
        const menuItems = menu
            ? [...menu.querySelectorAll('arc-menu-item')]
            : [];
        const firstMenuItem = menuItems[0];
        const lastMenuItem = menuItems[menuItems.length - 1];
        /*
        When up/down is pressed, we make the assumption that the user is familiar with the menu and plans to make a
        selection. Rather than toggle the panel, we focus on the menu (if one exists) and activate the first item for
        faster navigation.
        */
        if (['ArrowDown', 'ArrowUp'].includes(event.key)) {
            event.preventDefault();
            /* Show the menu if it's not already open */
            if (!this.open) {
                this.show();
            }
            /* Focus on a menu item */
            if (event.key === 'ArrowDown' && firstMenuItem) {
                menu.setCurrentItem(firstMenuItem);
                firstMenuItem.focus();
                return;
            }
            if (event.key === 'ArrowUp' && lastMenuItem) {
                menu.setCurrentItem(lastMenuItem);
                lastMenuItem.focus();
                return;
            }
        }
        /* Other keys bring focus to the menu and initiate type-to-select behavior */
        const ignoredKeys = ['Tab', 'Shift', 'Meta', 'Ctrl', 'Alt'];
        if (this.open && menu && !ignoredKeys.includes(event.key)) {
            menu.typeToSelect(event.key);
        }
    }
    /* Prevent space from triggering a click event in Firefox */
    handleTriggerKeyUp(event) {
        if (event.key === ' ') {
            event.preventDefault();
        }
    }
    handleTriggerClick() {
        if (this.open) {
            this.hide();
        }
        else {
            this.show();
        }
    }
    /* Hide the dropdown when a menu item is selected */
    handlePanelSelect(event) {
        const target = event.target;
        if (target.tagName === 'ARC-MENU') {
            this.hide();
            this.focusOnTrigger();
        }
    }
    render() {
        return html `
      <div
        id="main"
        class=${classMap({
            dropdown: true,
            'dropdown--open': this.open,
        })}
      >
        <span
          id="trigger"
          @click=${this.handleTriggerClick}
          @keydown=${this.handleTriggerKeyDown}
          @keyup=${this.handleTriggerKeyUp}
        >
          <slot
            id="triggerSlot"
            name="trigger"
            @slotchange=${this._updateAccessibleTrigger}
          ></slot>
        </span>

        <div id="positioner">
          <div
            id="panel"
            role="menu"
            aria-hidden=${this.open ? 'false' : 'true'}
            aria-labelledby="main"
            @arc-select=${this.handlePanelSelect}
          >
            <slot></slot>
          </div>
        </div>
      </div>
    `;
    }
}
/** @internal */
ArcDropdown.tag = 'arc-dropdown';
ArcDropdown.styles = styles;
__decorate([
    query('#trigger'),
    __metadata("design:type", HTMLElement)
], ArcDropdown.prototype, "trigger", void 0);
__decorate([
    query('#triggerSlot'),
    __metadata("design:type", HTMLSlotElement)
], ArcDropdown.prototype, "triggerSlot", void 0);
__decorate([
    query('#positioner'),
    __metadata("design:type", HTMLElement)
], ArcDropdown.prototype, "positioner", void 0);
__decorate([
    query('#panel'),
    __metadata("design:type", HTMLElement)
], ArcDropdown.prototype, "panel", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcDropdown.prototype, "placement", void 0);
__decorate([
    property({ attribute: false }),
    __metadata("design:type", HTMLElement)
], ArcDropdown.prototype, "containingElement", void 0);
__decorate([
    property({ type: Number }),
    __metadata("design:type", Number)
], ArcDropdown.prototype, "distance", void 0);
__decorate([
    property({ type: Number }),
    __metadata("design:type", Number)
], ArcDropdown.prototype, "skidding", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcDropdown.prototype, "open", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcDropdown.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcDropdown.prototype, "hoist", void 0);
__decorate([
    watch('open', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArcDropdown.prototype, "handleOpenChange", null);
__decorate([
    watch('distance'),
    watch('hoist'),
    watch('placement'),
    watch('skidding'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcDropdown.prototype, "handlePopoverOptionsChange", null);
__decorate([
    watch('disabled'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcDropdown.prototype, "handleDisabledChange", null);
setDefaultAnimation('dropdown.show', {
    keyframes: [
        { opacity: 0, transform: 'scale(0.9)' },
        { opacity: 1, transform: 'scale(1)' },
    ],
    options: ARC_ANIMATION_OPTIONS.fast,
});
setDefaultAnimation('dropdown.hide', {
    keyframes: [
        { opacity: 1, transform: 'scale(1)' },
        { opacity: 0, transform: 'scale(0.9)' },
    ],
    options: ARC_ANIMATION_OPTIONS.fast,
});
//# sourceMappingURL=ArcDropdown.js.map