import { __awaiter, __decorate, __metadata } from "tslib";
import { html, LitElement } from 'lit';
import { property, state, query } from 'lit/decorators.js';
import { classMap } from 'lit/directives/class-map.js';
import { styleMap } from 'lit/directives/style-map.js';
import { setDefaultAnimation, getAnimation, startAnimations, stopAnimations, parseDuration, } from '../../internal/animate.js';
import { emit } from '../../internal/event.js';
import { watch } from '../../internal/watch.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import styles from './arc-image.styles.js';
/**
 * @event arc-loaded - Emitted when the image is loaded.
 * @event arc-error - Emitted when the image cannot be loaded.
 *
 * @ssr - True
 */
export default class ArcImage extends LitElement {
    constructor() {
        super(...arguments);
        /** @internal - Options that define certain aspects of the observer’s behavior. */
        this._observerOptions = {
            root: null,
            rootMargin: '0px',
            threshold: [0, 1],
        };
        /** @internal - Used to set a timeout on loading the image. */
        this._loadTimer = null;
        /** @internal - Loading state of the image. */
        this._loading = false;
        /** @internal - Is a valid image loaded?. */
        this._hasImage = false;
        /** Set the delay in ms before loading the image. */
        this.delay = 1000;
    }
    handleSrcChange() {
        this._attachObserver();
    }
    handleLoadingChange() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._loading) {
                yield stopAnimations(this.loader);
                const { keyframes, options } = getAnimation(this.loader, 'loader.show');
                yield startAnimations(this.loader, keyframes, options);
            }
            else {
                yield stopAnimations(this.loader);
            }
        });
    }
    firstUpdated() {
        this._attachObserver();
    }
    _attachObserver() {
        this._removeObserver();
        this._intersectionObserver = new IntersectionObserver(this._handleIntersection.bind(this), this._observerOptions);
        this._intersectionObserver.observe(this.container);
    }
    _removeObserver() {
        if (!this._intersectionObserver)
            return;
        this._intersectionObserver.disconnect();
    }
    /* Callback that gets fired whenever the first or last pixel of the container is intersecting the root element */
    _handleIntersection(entries) {
        entries.forEach(({ intersectionRatio }) => {
            if (intersectionRatio === 0) {
                if (this._loadTimer) {
                    clearTimeout(this._loadTimer);
                    this._loadTimer = null;
                }
            }
            else if (intersectionRatio === 1) {
                this._loadTimer = window.setTimeout(this._loadImage.bind(this), this.delay);
            }
        });
    }
    /* Load the image and handle the response */
    _loadImage() {
        this._loading = true;
        this.image.src = this.src;
        this.image.onload = this._imageResponse.bind(this);
        this.image.onerror = this._imageResponse.bind(this);
    }
    /* Emit the response from the image */
    _imageResponse(e) {
        this._loading = false;
        this._hasImage = e.type === 'load';
        this._removeObserver();
        emit(this, this._hasImage ? ARC_EVENTS.loaded : ARC_EVENTS.error, {
            detail: e,
        });
    }
    handleSize(size) {
        if (size.endsWith('px') || size.endsWith('%')) {
            return size;
        }
        return `${size}px`;
    }
    render() {
        const imageStyles = {
            width: this.width ? this.handleSize(this.width) : undefined,
            height: this.height ? this.handleSize(this.height) : undefined,
        };
        return html `
      <div
        id="main"
        class=${classMap({
            image: true,
            'image--has-image': this._hasImage,
            'image--loading': this._loading,
        })}
        style=${styleMap(imageStyles)}
      >
        <img id="image" src="" alt=${this.alt} />
        <div id="overlay">
          <div id="loader"></div>
          <svg
            id="placeholder"
            width="100%"
            height="100%"
            viewBox="0 0 360 172"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
          >
            <path
              d="M182.36 55L216.446 114.04H148.273L182.36 55Z"
              fill="rgb(var(--arc-grey-030))"
            />
            <circle cx="236" cy="55" r="17" fill="rgb(var(--arc-grey-030))" />
            <rect
              x="122"
              y="87"
              width="27"
              height="27"
              fill="rgb(var(--arc-grey-030))"
            />
          </svg>
        </div>
      </div>
    `;
    }
}
/** @internal */
ArcImage.tag = 'arc-image';
ArcImage.styles = styles;
__decorate([
    query('#main'),
    __metadata("design:type", HTMLElement)
], ArcImage.prototype, "container", void 0);
__decorate([
    query('#image'),
    __metadata("design:type", HTMLImageElement)
], ArcImage.prototype, "image", void 0);
__decorate([
    query('#loader'),
    __metadata("design:type", HTMLElement)
], ArcImage.prototype, "loader", void 0);
__decorate([
    state(),
    __metadata("design:type", Boolean)
], ArcImage.prototype, "_loading", void 0);
__decorate([
    state(),
    __metadata("design:type", Boolean)
], ArcImage.prototype, "_hasImage", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcImage.prototype, "src", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcImage.prototype, "alt", void 0);
__decorate([
    property({
        type: Number,
        converter: (attrValue) => attrValue ? parseDuration(attrValue) : 1000,
    }),
    __metadata("design:type", Number)
], ArcImage.prototype, "delay", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcImage.prototype, "width", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcImage.prototype, "height", void 0);
__decorate([
    watch('src', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcImage.prototype, "handleSrcChange", null);
__decorate([
    watch('_loading', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArcImage.prototype, "handleLoadingChange", null);
setDefaultAnimation('loader.show', {
    keyframes: [
        {
            boxShadow: 'var(--shadow-n), var(--shadow-n), var(--shadow-n), var(--shadow-n)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-pn), var(--shadow-pn), var(--shadow-pn)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-pn), var(--shadow-p), var(--shadow-p)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-pn), var(--shadow-p), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-pn), var(--shadow-p), var(--shadow-n)',
        },
        {
            boxShadow: 'var(--shadow-pn), var(--shadow-pn), var(--shadow-p), var(--shadow-pn)',
        },
        {
            boxShadow: 'var(--shadow-p), var(--shadow-p), var(--shadow-p), var(--shadow-p)',
        },
        {
            boxShadow: 'var(--shadow-np), var(--shadow-np), var(--shadow-p), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-n), var(--shadow-p), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-pn), var(--shadow-p), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-p), var(--shadow-p), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-np), var(--shadow-np), var(--shadow-np)',
        },
        {
            boxShadow: 'var(--shadow-n), var(--shadow-n), var(--shadow-n), var(--shadow-n)',
        },
    ],
    options: {
        duration: 6000,
        iterations: Infinity,
    },
});
//# sourceMappingURL=ArcImage.js.map