import { __awaiter, __decorate, __metadata } from "tslib";
import { html, isServer, LitElement } from 'lit';
import { property, state } from 'lit/decorators.js';
import { until } from 'lit/directives/until.js';
import * as Msal from '@azure/msal-browser';
import { PublicClientApplication, EventType, } from '@azure/msal-browser';
import { emit } from '../../internal/event.js';
import { watch } from '../../internal/watch.js';
import { stringToArray } from '../../internal/string.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import { FLOATING_PLACEMENTS } from '../../internal/constants/placementConstants.js';
import styles from './arc-sso.styles.js';
import '../button/arc-button.js';
import '../avatar/arc-avatar.js';
import '../dropdown/arc-dropdown.js';
import '../menu/arc-menu.js';
import '../menu-item/arc-menu-item.js';
/**
 * @slot login - Used to overwrite the default login slot.
 * @slot logout - Used to overwrite the default logout slot.
 *
 * @event arc-auth - Emitted when the internal authentication state of the component changes.
 *
 * @ssr - True
 */
export default class ArcSSO extends LitElement {
    constructor() {
        super(...arguments);
        /** @internal
        openid - By using this permission, an app can receive a unique identifier for the user in the form of the sub claim.
        The permission also gives the app access to the UserInfo endpoint.
        The `openid` scope can be used at the Microsoft identity platform token endpoint to acquire ID tokens.
        The app can use these tokens for authentication.
      
        profile - The `profile` scope can be used with the openid scope and any other scope.
        It gives the app access to a large amount of information about the user.
        The information it can access includes, but isn't limited to, the user's given name, surname, preferred username, and object ID.
      
        User.Read - The User.Read permission gives access to the sub claim.
        It allows the client or app to correctly identify the user over time and access rudimentary user information.
        */
        this.loginRequest = {
            scopes: ['openid', 'profile', 'User.Read'],
        };
        /** @internal - State that keeps track of the auth status of the user. */
        this._isAuth = false;
    }
    handleAuthChange() {
        return __awaiter(this, void 0, void 0, function* () {
            emit(this, ARC_EVENTS.auth, {
                detail: {
                    authenticated: this._isAuth,
                    account: this.getAccount(),
                },
            });
            /* c8 ignore next 3 */
            if (this._isAuth) {
                this._avatar = yield this.getAvatar();
            }
        });
    }
    connectedCallback() {
        super.connectedCallback();
        /* Initialize the MSAL authentication context */
        if (!this.msalInstance) {
            this.msalInstance = this._initMsal();
        }
        /* Add additional scopes (permissions) */
        if (this.scopes && this.scopes.length > 0) {
            this.loginRequest.scopes = this.loginRequest.scopes.concat(this.scopes);
        }
        /* If rendered on server dont get active account */
        if (isServer)
            return;
        this._callbackId = this.msalInstance.addEventCallback(this._handleAuthChange.bind(this));
        /* Update the _isAuth state */
        this._isAuth = !!this.getAccount();
    }
    _handleAuthChange(event) {
        if (event.eventType === EventType.LOGIN_SUCCESS && !!event.payload) {
            this.msalInstance.setActiveAccount(event.payload);
            this._isAuth = true;
        }
        else if (event.eventType === EventType.LOGOUT_SUCCESS) {
            this._isAuth = false;
        }
    }
    updated(changedProperties) {
        super.updated(changedProperties);
        if (changedProperties.has('msalInstance')) {
            // If msalInstance has just been set
            const oldMsal = changedProperties.get('msalInstance');
            if (this._callbackId && oldMsal) {
                // If there was a previously registered instance
                oldMsal.removeEventCallback(this._callbackId);
            }
            this._callbackId = this.msalInstance.addEventCallback(this._handleAuthChange.bind(this));
        }
        else if (changedProperties.has('clientId') ||
            changedProperties.has('tenantId') ||
            changedProperties.has('redirectUri')) {
            // If properties requiring a new instance have been set
            if (this._callbackId) {
                this.msalInstance.removeEventCallback(this._callbackId);
            }
            this.msalInstance = this._initMsal();
            this._callbackId = this.msalInstance.addEventCallback(this._handleAuthChange.bind(this));
        }
    }
    /* Initialize the MSAL authentication context */
    _initMsal() {
        const msalConfig = {
            auth: {
                clientId: this.clientId,
                authority: `https://login.microsoftonline.com/${this.tenantId ? this.tenantId : 'common/'}`,
                redirectUri: this.redirectUri,
            },
            cache: {
                cacheLocation: 'localStorage',
                storeAuthStateInCookie: false,
            },
        };
        /* c8 ignore next 3 */
        return new Msal.PublicClientApplication(msalConfig);
    }
    _getAccessToken() {
        if (isServer)
            return undefined;
        const account = this.getAccount();
        const accessTokenRequest = {
            account,
            scopes: this.loginRequest.scopes,
        };
        /* c8 ignore next 5 */
        return account
            ? this.msalInstance
                .acquireTokenSilent(accessTokenRequest)
                .then((resp) => resp.accessToken)
            : undefined;
    }
    /* c8 ignore next 19 */
    signIn() {
        this.msalInstance.handleRedirectPromise().then(() => {
            /* Check if user signed in */
            if (!this.getAccount()) {
                this.msalInstance.loginPopup(this.loginRequest);
            }
        });
    }
    /* c8 ignore next 4 */
    signOut() {
        this.msalInstance.logoutRedirect();
    }
    getAccount() {
        if (isServer)
            return;
        return this.msalInstance.getAllAccounts()[0];
    }
    getAvatar() {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield this._getAccessToken();
            /* c8 ignore next 9 */
            return token
                ? fetch('https://graph.microsoft.com/v1.0/me/photos/48x48/$value', {
                    headers: {
                        Authorization: `Bearer ${token}`,
                        'Content-Type': 'image/jpg',
                    },
                })
                    .then((resp) => resp.blob())
                    .then((resp) => URL.createObjectURL(resp))
                : '';
        });
    }
    render() {
        const { name } = this.getAccount() || {};
        return html `
      <div id="main">
        <slot name="login" ?hidden=${this._isAuth}>
          <arc-button type="tab" @click=${this.signIn}>Login</arc-button>
        </slot>
        <slot name="logout" ?hidden=${!this._isAuth}>
          <arc-dropdown
            id="userMenu"
            placement=${FLOATING_PLACEMENTS['bottom-end']}
            hoist
          >
            <arc-button id="desktopTrigger" slot="trigger" type="tab">
              ${name}
              <arc-avatar
                slot="suffix"
                name=${name}
                image=${until(this._avatar, '')}
                label="User avatar"
              ></arc-avatar>
            </arc-button>
            <arc-avatar
              id="mobileTrigger"
              slot="trigger"
              image=${until(this._avatar, '')}
              name=${name}
              label="User avatar"
            ></arc-avatar>
            <arc-menu>
              <arc-menu-item @click=${this.signOut}>Logout</arc-menu-item>
            </arc-menu>
          </arc-dropdown>
        </slot>
      </div>
    `;
    }
}
/** @internal */
ArcSSO.tag = 'arc-sso';
ArcSSO.styles = styles;
__decorate([
    property({ type: PublicClientApplication, attribute: false }),
    __metadata("design:type", PublicClientApplication)
], ArcSSO.prototype, "msalInstance", void 0);
__decorate([
    state(),
    __metadata("design:type", Boolean)
], ArcSSO.prototype, "_isAuth", void 0);
__decorate([
    state(),
    __metadata("design:type", String)
], ArcSSO.prototype, "_avatar", void 0);
__decorate([
    property({ type: String, attribute: 'client-id' }),
    __metadata("design:type", String)
], ArcSSO.prototype, "clientId", void 0);
__decorate([
    property({ type: String, attribute: 'tenant-id' }),
    __metadata("design:type", String)
], ArcSSO.prototype, "tenantId", void 0);
__decorate([
    property({ type: String, attribute: 'redirect-uri' }),
    __metadata("design:type", String)
], ArcSSO.prototype, "redirectUri", void 0);
__decorate([
    property({
        type: Array,
        converter: (attrValue) => attrValue ? stringToArray(attrValue) : [],
    }),
    __metadata("design:type", String)
], ArcSSO.prototype, "scopes", void 0);
__decorate([
    watch('_isAuth'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArcSSO.prototype, "handleAuthChange", null);
//# sourceMappingURL=ArcSSO.js.map