import { __decorate, __metadata } from "tslib";
import { html, LitElement } from 'lit';
import { property, query } from 'lit/decorators.js';
import { ifDefined } from 'lit/directives/if-defined.js';
import { live } from 'lit/directives/live.js';
import { classMap } from 'lit/directives/class-map.js';
import { emit } from '../../internal/event.js';
import { FormController } from '../../internal/form-control.js';
import { watch } from '../../internal/watch.js';
import { ARC_EVENTS } from '../../internal/constants/eventConstants.js';
import styles from './arc-switch.styles.js';
/**
 * @slot default - The checkbox' label.
 *
 * @event arc-change - Emitted when the control's checked state changes.
 *
 * @ssr - True
 */
export default class ArcSwitch extends LitElement {
    constructor() {
        super(...arguments);
        /** @internal - Controller used to recognize form controls located inside a shadow root. */
        /* @ts-expect-error - Controller used to hook the component to the formData */
        this.formController = new FormController(this, {
            value: (control) => control.checked ? control.value : undefined,
        });
        /** Draws the component in a disabled state. */
        this.disabled = false;
        /** Makes the switch a required field. */
        this.required = false;
        /** Draws the component in a checked state. */
        this.checked = false;
        /** This will be true when the control is in an invalid state. Validity is determined by the `required` prop. */
        this.invalid = false;
    }
    /* Enable/disable the editor when the disabled property changes */
    handleDisabledChange() {
        /* Disabled form controls are always valid, so we need to recheck validity when the state changes */
        this.input.disabled = this.disabled;
        this.invalid = !this.input.checkValidity();
    }
    handleCheckedChange() {
        this.input.checked = this.checked;
        this.invalid = !this.input.checkValidity();
    }
    firstUpdated() {
        this.invalid = !this.input.checkValidity();
    }
    /* Simulates a click on the switch. */
    click() {
        this.input.click();
    }
    /* Sets focus on the switch. */
    focus(options) {
        this.input.focus(options);
    }
    /* Removes focus from the switch. */
    blur() {
        this.input.blur();
    }
    /** Checks for validity and shows the browser's validation message if the control is invalid. */
    reportValidity() {
        return this.input.reportValidity();
    }
    /** Sets a custom validation message. If `message` is not empty, the field will be considered invalid. */
    setCustomValidity(message) {
        this.input.setCustomValidity(message);
        this.invalid = !this.input.checkValidity();
    }
    /* Handle the click of the switch */
    _handleClick() {
        this.checked = !this.checked;
        emit(this, ARC_EVENTS.change);
    }
    render() {
        return html `
      <label
        id="main"
        class=${classMap({
            switch: true,
            'switch--checked': this.checked,
            'switch--disabled': this.disabled,
        })}
      >
        <span id="base">
          <input
            type="checkbox"
            name=${ifDefined(this.name)}
            .value=${ifDefined(this.value)}
            .checked=${live(this.checked)}
            .disabled=${this.disabled}
            .required=${this.required}
            role="switch"
            aria-checked=${this.checked ? 'true' : 'false'}
            aria-disabled=${this.disabled ? 'true' : 'false'}
            @click=${this._handleClick}
          />
          <span id="control">
            <span id="thumb"></span>
          </span>
        </span>
        <span id="label"><slot></slot></span>
      </label>
    `;
    }
}
/** @internal */
ArcSwitch.tag = 'arc-switch';
ArcSwitch.styles = styles;
__decorate([
    query('input[type="checkbox"]'),
    __metadata("design:type", HTMLInputElement)
], ArcSwitch.prototype, "input", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcSwitch.prototype, "name", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], ArcSwitch.prototype, "value", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcSwitch.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Object)
], ArcSwitch.prototype, "required", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Boolean)
], ArcSwitch.prototype, "checked", void 0);
__decorate([
    property({ type: Boolean, reflect: true }),
    __metadata("design:type", Object)
], ArcSwitch.prototype, "invalid", void 0);
__decorate([
    watch('disabled', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcSwitch.prototype, "handleDisabledChange", null);
__decorate([
    watch('checked', { waitUntilFirstUpdate: true }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ArcSwitch.prototype, "handleCheckedChange", null);
//# sourceMappingURL=ArcSwitch.js.map